#ifndef LOCARNA_AUX_HH
#define LOCARNA_AUX_HH

#include <exception>
#include <string>


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

namespace LocARNA {

    /** 
     * @brief Function class definining hash function for pairs of size_t
     */
    struct pair_of_size_t_hash
    {
	/** 
	 * @brief Hash function for pairs of size_t
	 * 
	 * @return hash code
	 */
	size_t
	operator()(std::pair<size_t,size_t> p) const { 
	    return p.first<<(sizeof(size_t)/2) | p.second;
	}
    };

    //! Simple exception class that supports a text message
    class failure : public std::exception {
        //! message that is reported by what
        std::string msg_;
    public:
        /** 
         * @brief Construct with message
         * 
         * @param msg the message
         */
        explicit
        failure (const std::string& msg): std::exception(), msg_(msg) {};
        
        /** 
         * @brief Construct empty
         */
        explicit 
        failure (): std::exception(), msg_() {};
        
        //! Destruct
        virtual
        ~failure() throw();
        
        /** @brief Provide message string
         * @return message
         */
        virtual
        const char* what() const throw();
    };

   
}


#endif
