#include <iostream>
#include <fstream>
#include <string>

#include <assert.h>

#include <mea_eval_cmdline.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mea_predictor.hh"
#include "mea_evaluator.hh"


int
main(int argc, char *argv[])
{
  gengetopt_args_info args_info;
  
  if (cmdline_parser(argc,argv,&args_info) != 0) {
    return -1;
  }
  
  if (args_info.inputs_num!=1) {
    cmdline_parser_print_help();
    cmdline_parser_free(&args_info);
    exit(-1);
  }
  
  std::string filename = args_info.inputs[0];
  std::string dpdir    = args_info.dpdir_arg;
  std::string pred     = args_info.pred_arg;

  bool slide_rule    = !args_info.no_slide_rule_given;
  bool conflict_rule = !args_info.no_conflict_rule_given;


  if(dpdir.empty() && pred.empty() || !dpdir.empty() && !pred.empty() ) //XOR
    {
      std::cerr << "give either filename or path as input (-dpdir or -pred)" << std::endl;
      return -1;
    }
  
  LocARNA::ConfusionMatrix::BPFilter *filter = new LocARNA::ConfusionMatrix::BPFilter();
  
  if (args_info.start_given || args_info.stop_given) {
      if (args_info.stop_arg>args_info.start_arg
	  ||
	  args_info.stop_arg==0) {
	  
	  delete filter;
	  filter 
	      = new LocARNA::ConfusionMatrix::SpanRangeBPFilter(args_info.start_arg,
								args_info.stop_arg);
      } else {
	  std::cerr << "Invalid base pair range." << std::endl;
	  return -1;
      }
  }

  try {
      
      MeaEvaluator me(filename,dpdir,pred,slide_rule,conflict_rule,filter);
        
    double f_score =
      me.evaluate(args_info.alpha_arg,
		  args_info.beta_arg,
		  args_info.gamma_arg,
		  args_info.delta_arg);
      
    std::cout << "F1-score: " << f_score << std::endl;
    
  } catch (std::ios_base::failure f) {
      std::cerr <<f.what()<<std::endl; 
      cmdline_parser_free(&args_info);
      delete filter;
      return -1;
  }

  delete filter;

  cmdline_parser_free(&args_info);
  return 0;
}
