#include <iostream>
#include <fstream>
#include <string>

#include <assert.h>

#include "mea_mix_cmdline.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "LocARNA/aux.hh"
#include "LocARNA/confusion_matrix.hh"
#include "LocARNA/rna_structure.hh"

#include "mea_predictor.hh"

typedef MeaMixDPsPredictor<MeaOSPredictor> predictor_t;

int
main(int argc, char *argv[])
{
  gengetopt_args_info args_info;
  
  if (cmdline_parser(argc,argv,&args_info) != 0) {
    return -1;
  }
  
  if (args_info.inputs_num!=2) {
    cmdline_parser_print_help();
    cmdline_parser_free(&args_info);
    exit(-1);
  }
  
  std::string filename = args_info.inputs[0];
  std::string filename2 = args_info.inputs[1];

  bool slide_rule    = !args_info.no_slide_rule_given;
  bool conflict_rule = !args_info.no_conflict_rule_given;


  
  try {
    std::ifstream in(filename.c_str());
      
    if (! in.good()) {
      throw std::ios_base::failure("Cannot open "+filename+" for reading.");
    }

    std::ifstream in2(filename2.c_str());
      
    if (! in2.good()) {
      throw std::ios_base::failure("Cannot open "+filename+" for reading.");
    }
    
      predictor_t 
	mp(in, 
	   in2,
	   args_info.gamma1_arg,
	   args_info.gamma2_arg);
      
      in.close();


      double score = mp.fold(); // dp
      //traceback
      std::string predicted_structure = mp.traceback();
      
      std::cout << mp.sequence() << std::endl;
      std::cout << predicted_structure << " (" << score <<")" << std::endl;
      
      if (args_info.reference_given) {
	  const std::string reference_structure = args_info.reference_arg;
	  if (reference_structure.length() != mp.sequence().length()) {
	      std::cerr << "Failure: length of sequence and refernence differ."
			<< std::endl;
	      return -1;      
	  }
	  
	  try {
	    LocARNA::RnaStructure refstruct(reference_structure);
	    LocARNA::RnaStructure predstruct(predicted_structure);
	    LocARNA::ConfusionMatrix cm(refstruct,predstruct,slide_rule,conflict_rule);
	      
	      // write TP FP FN TN SENS PPV F1 MCC
	      std::cout << cm.tp() << " "
			<< cm.fp() << " "
			<< cm.fn() << " "
			<< cm.tn() << " "
			<< cm.sens() << " "
			<< cm.ppv() << " "
			<< cm.f1_score() << " "
			<< cm.mcc() << std::endl;
	      
	  } catch (LocARNA::failure f) {
	      std::cerr << "Failure: "<<f.what()<<std::endl;
	      cmdline_parser_free(&args_info);
	      return -1;
	  }
	  
      }
      
  } catch (std::ios_base::failure f) {
      std::cerr <<f.what()<<std::endl; 
      cmdline_parser_free(&args_info);
      return -1;
  }
  
  cmdline_parser_free(&args_info);
  return 0;
}
