#include "mea_predictor.hh" 

void
MeaPredictorBase::comp_unpaired_probs(const pair_probs_t &p, unpaired_probs_t &pu)
{
  //compute unproperties =  property for base not binding
  pu.resize(length_,1.0);
    
  for(unsigned int i=0;i<length_;i++)
    {
      for(unsigned int j=0;j<i;j++)//j>i
	{
	  pu[i] -= p(j,i);
	}
      for(unsigned int j=i+1;j<length_;j++)//j<i
	{
	  pu[i] -= p(i,j);	    
	}
    }
}

void 
MeaPredictorBase::readDotPlot(std::istream &in, std::string &sequence, pair_probs_t& pmat)
{
  pmat.clear();
  std::string sLine;
    
  bool have_read_seq = false;
    
  while (!in.eof())
    {
      getline(in,sLine);
	
      //read sequence
      if(sLine.substr(0,9) == "/sequence")
	{
	  std::string s;
	  std::string temp;
	  getline(in,temp);	      
	  while(temp[0]!=')')
	    {	    
	      s = s.append(temp.replace(temp.size()-1,1,""));
	      std::getline(in,temp);
	    }
	    
	  sequence = s;
	  //int len = sequence.size();
	    
	  have_read_seq = true;
	}
	
      //fill seq. structure vector
      if(sLine.find("ubox")!= -1 && sLine.substr(0,1)!="%" && have_read_seq )
	{
	  std::string buf; // Have a buffer string
	  std::stringstream ss(sLine); // Insert the string into a stream
	    
	  std::vector<std::string> tokens; // Create vector to hold our words

	  int i;
	  int j;
	  double sqrt_p;
	  
	  if (ss >> i >> j >> sqrt_p) {
	    pmat(i-1,j-1)=sqrt_p*sqrt_p;
	  } else {
	    std::cout << " strange ubox entry" << std::endl;
	  }
	}
    }
}

double
MeaCLPredictor::fold()
{
  dp_.resize(length());
    
  for(size_t i=0;i<length();i++)
    {
      dp_[i].resize(length());
      dp_[i][i]=unpaired_score(i);
    }
  
  //fill matrix
  for(size_t j=0;j<length();j++)
    {
      std::vector<size_t> candidates;
	
      for(size_t i=j;i>0;)
	{
	  --i;
	  score_t cur = dp_[i][j-1]   + unpaired_score(j);
	  //  for(size_t k=i+1; k+min_loop_size<j; ++k) 
	  for(std::vector<size_t>::const_iterator k=candidates.begin();
	      candidates.end()!=k; ++k) 
	    {
	      cur = std::max(cur, dp_[i][*k-1] + dp_[*k][j] );
	    }
	    
	  if (i+min_loop_size()<j) {
	    int score_ct = dp_[i+1][j-1] + bp_score(i,j);
	    if (score_ct > cur) {
	      cur=score_ct;
	      candidates.push_back(i);
	    }
	  }
	  dp_[i][j] = cur;
	}
    }

  // print_matrix_debug();
  
  return (double)dp_[0][length()-1]/(double)PSCALE();
}

std::string
MeaCLPredictor::traceback() {
  structure_.resize(length(),'.');
  traceback(0,length()-1);
  return structure_;
}


void
MeaCLPredictor::traceback(int i, int j)
{
  if (i+min_loop_size()+1 > j) return;

  if (dp_[i][j] == dp_[i][j-1] + unpaired_score(j))
    {
      traceback(i,j-1);
      return;
    }
    
  for(int k=i+1;k+min_loop_size()<j;k++)
    {
      // cout << k << endl;
      if (dp_[i][j] == dp_[i][k-1] + dp_[k][j]) {
	traceback(i,k-1);
	traceback(k,j);
	return;
      }
    }
        
  if ((i+min_loop_size()<j) && dp_[i][j] == dp_[i+1][j-1] + bp_score(i,j)) 
    {
      structure_[i]='(';
      structure_[j]=')';
	
      traceback(i+1,j-1);
      return;
    }
    
  std::cerr <<"Something went wrong!"<<std::endl;
  assert(false);
}

void
MeaCLPredictor::print_matrix_debug() const {
    //print matrix
    for(unsigned int i=0;i<dp_.size();i++)
      {
	for(unsigned int j = 0;j<dp_[i].size();j++)
	  {
	    std::cout << dp_[i][j] << "\t";
	  }
	std::cout << "" << std::endl;
      }
  }


double
MeaOSPredictor::fold() {
  row_i_.resize(length());
  row_ip1_.resize(length());

  octs_.resize(length());
  
  for (size_t i=length(); i>0;) { i--;
    //rotate row i to row i+1 and make space for new row i
    row_ip1_.swap(row_i_);

    // singletons are oct, but we handle them in extra cases
    // therefore don't do: octs_[i].push_back(std::pair<size_t,score_t>(i,unpaired_score(i)));
    
    // init row i
    row_i_[i]=unpaired_score(i);
    for (size_t j=i+1; j<length(); j++) {
      row_i_[j] = 
	unpaired_score(i) + row_ip1_[j]; // decompose i # i+1..j
    }
    
    // for (size_t j=0; j<i; j++) {
    //   std::cout << 0 << "\t";
    // }
    // std::cout << row_i_[i] << "\t";
    
    // 
    for (size_t j=i+1; j<length(); j++) {
      
      if (i+min_loop_size()<j) {
	score_t  oct_val = row_ip1_[j-1] + bp_score(i,j);   // decompose ( i+1..j-1 )
	if ( oct_val > row_i_[j] ) {
	  row_i_[j] = oct_val;
	  octs_[i].push_back(std::pair<size_t,score_t>(j,oct_val));
	}
      }
      
      // std::cout << row_i_[j] << "\t";
      
      if (j+1<length()
	  && row_i_[j] > unpaired_score(i) + row_ip1_[j] ) { // if i..j is a STEP
	
	// decompose i..j # j+1 
	row_i_[j+1] = std::max(row_i_[j+1], row_i_[j] + unpaired_score(j+1) );
	
	// decompose i..j # j+1..k for all k:oct(j+1,k)
	for (oct_vector_t::const_iterator oct=octs_[j+1].begin();
	     octs_[j+1].end()!=oct; ++oct) {
	  
	  size_t k    = oct->first; //right end of oct
	  row_i_[k] = std::max(row_i_[k], row_i_[j] + oct->second);
	}
      }
    }
    // std::cout << std::endl;
  }

  //print_matrix_debug();
  return (double)row_i_[length()-1]/(double)PSCALE();
}

void MeaOSPredictor::print_matrix_debug() const {
  for (size_t i=0; i<length(); i++) {
    std::cout << i << " -- ";    
    for (std::vector<std::pair<size_t, score_t> >::const_iterator it=octs_[i].begin();
	 octs_[i].end()!=it; ++it) {
      std::cout << it->first << ":" << it->second << " ";
    }
    std::cout << std::endl;
  }
}

std::string
MeaOSPredictor::traceback() {
  restore(0,length()-1);
  structure_.resize(length(),'.');
  traceback(0,length()-1);
  return structure_;
}

// restore in row i
void
MeaOSPredictor::restore(size_t i,size_t j) {
  std::vector<score_t> &col_j=row_i_;

  col_j[j]=unpaired_score(j);  
  for (size_t ip=j; ip>i; ) { ip--; // for i' from j-1 downto i
    if (is_oct(ip,j)) {
      col_j[ip] = get_oct_score(ip,j);
    } else {
      col_j[ip]=0;
      // decompose i'..q q+1..j, where i'..q is OCT
      for (oct_vector_t::const_iterator oct=octs_[ip].begin();
	   octs_[ip].end()!=oct && oct->first<j; ++oct) {
	
	col_j[ip] = std::max(col_j[ip], oct->second + col_j[oct->first+1]);
      }
      col_j[ip] = std::max(col_j[ip], unpaired_score(ip) + col_j[ip+1]);
    }
  }
}

void
MeaOSPredictor::traceback(size_t i,size_t j) {
  std::vector<score_t> &col_j=row_i_;

  if (i+min_loop_size() >= j) return;
  
  // pre: entries col_j[i..j] are restored 
  
  // find optimum decomposition i..q # q+1..j where i..q is OCT
  
  if (i!=j && is_oct(i,j)) {
    structure_[i]='(';
    structure_[j]=')';
    restore(i+1,j-1);
    traceback(i+1,j-1);
  } else {
    if (col_j[i] == unpaired_score(i) + col_j[i+1]) {
      traceback(i+1,j);
    } else {
      for (oct_vector_t::const_iterator oct=octs_[i].begin();
	   octs_[i].end()!=oct && oct->first<j; ++oct) {
	const size_t &q=oct->first;
	if (col_j[i] == oct->second + col_j[q+1]) {
	  traceback(q+1,j); // first trace back in the same column j
	  
	  // i..q is oct and  i!=q
	  
	  // output a pair
	  structure_[i]='(';
	  structure_[q]=')';
	  
	  // ... and second, restore entries and trace back in column q-1
	  restore(i+1,q-1);
	  traceback(i+1,q-1);
	  
	  break; // don't search for optimum decomposition any further
	}
      }
    }
  }
  
  return;
}
